@@ -2,7 +2,7 @@
 #' @description Table for coxph.object with model option: TRUE - allow "frailty" or "cluster" model
 #' @param cox.obj.withmodel coxph.object with model option: TRUE
 #' @param dec Decimal point, Default: 2
-#' @param msm Multi state model, Default: NULL
+#' @param event_msm Character or numeric vector of destination states to keep (multi-state models only)
 #' @param pcut.univariate pcut.univariate, Default: NULL
 #' @param data_for_univariate data for univariate model, Default: NULL
 #' @return Table, cluster/frailty info, metrics, caption
@@ -23,7 +23,7 @@
 #'
 
 
-cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate = NULL, data_for_univariate = NULL) {
+cox2.display <- function(cox.obj.withmodel, dec = 2, event_msm = NULL, pcut.univariate = NULL, data_for_univariate = NULL) {
   if (!is.null(data_for_univariate) && !data.table::is.data.table(data_for_univariate)) {
     data_for_univariate <- data.table::as.data.table(data_for_univariate)
   }
@@ -31,6 +31,22 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
   if (!any(class(model) == "coxph")) {
     stop("Model not from Cox model")
   }
+  model_states <- model$states
+  if (!is.null(model_states)) {
+    model_states <- as.character(model_states)
+    allowed_idx    <- which(model_states != "(s0)")
+    allowed_labels <- model_states[allowed_idx]
+  } else {
+    allowed_idx <- integer()
+    allowed_labels <- character()
+  }
+  
+  if (is.null(model_states) && !is.null(event_msm)) {
+    stop("'event_msm' can only be used when the model includes multi-state information")
+  }
+  filtered_state_labels <- NULL
+  rn.uni_filtered <- NULL
+  use_event_filter <- FALSE
   xf <- attr(model$terms, "term.labels") # Independent vars
   xf_keep <- xf
   xf.old <- xf
@@ -142,6 +158,79 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
   #  mdata = data.frame(data)
   # }
   
+  target_state_idx <- integer()
+  target_state_labels <- character()
+
+  if (!is.null(model_states)) {
+    if (!is.null(event_msm)) {
+      msm_values <- as.vector(event_msm)
+      if (length(msm_values) == 0 || all(is.na(msm_values))) {
+        stop("event_msm must contain at least one non-missing value.")
+      }
+
+      if (length(allowed_idx) == 0) {
+        stop("The model does not contain any destination states beyond the origin state '(s0)'.")
+      }
+
+      available_states <- paste(sprintf("%d: %s", allowed_idx, allowed_labels), collapse = ", ")
+
+      resolve_state <- function(val) {
+        if (is.na(val)) return(NA_integer_)
+        if (is.numeric(val)) {
+          if (!is.finite(val)) return(NA_integer_)
+          if (!isTRUE(all.equal(val, as.integer(val)))) return(NA_integer_)
+          idx <- as.integer(val)
+          if (idx %in% allowed_idx) return(idx)
+          return(NA_integer_)
+        }
+        val_chr <- trimws(as.character(val))
+        if (identical(val_chr, "") || identical(val_chr, "(s0)")) return(NA_integer_)
+        match_idx <- match(val_chr, model_states)
+        if (!is.na(match_idx) && match_idx %in% allowed_idx) return(match_idx)
+        suppressWarnings(idx_chr <- as.integer(val_chr))
+        if (!is.na(idx_chr) && idx_chr %in% allowed_idx) return(idx_chr)
+        NA_integer_
+      }
+
+      single_destination <- length(allowed_idx) == 1
+      if (single_destination) {
+        allowed_label <- allowed_labels[1]
+        valid_tokens  <- unique(c(allowed_label, as.character(allowed_idx)))
+
+        if (length(msm_values) > 1) {
+          stop(sprintf("Only one destination state ('%s') is available; event_msm cannot contain multiple values.",
+                       allowed_label))
+        }
+
+        if (!all(msm_values %in% valid_tokens)) {
+          baseline_label <- model_states[1]
+          stop(sprintf(
+            "event_msm must match the available destination state '%s'. Baseline states such as '%s' are not allowed.",
+            allowed_label, baseline_label))
+        }
+
+        target_state_idx    <- allowed_idx
+        target_state_labels <- allowed_label
+      } else {
+        state_idx_all <- vapply(msm_values, resolve_state, integer(1), USE.NAMES = FALSE)
+        if (anyNA(state_idx_all)) {
+          bad_vals <- unique(as.character(msm_values[is.na(state_idx_all)]))
+          stop(sprintf(
+            "No valid states matched 'event_msm'. event_msm must refer to destination states reported by model$states (excluding '%s'). Available states: %s",
+            model_states[1], available_states))
+        }
+        state_idx <- unique(state_idx_all)
+
+        target_state_idx    <- state_idx
+        target_state_labels <- model_states[state_idx]
+      }
+    }
+  } else if (!is.null(event_msm)) {
+    stop("'event_msm' can only be used when the model includes multi-state information")
+  }
+
+  use_event_filter <- length(target_state_idx) > 0 && length(allowed_idx) > 1
+
   if (length(xf) == 1) {
     uni.res <- data.frame(summary(model)$coefficients)
     # uni.res <- data.frame(summary(coxph(as.formula(paste("mdata[, 1]", "~", xf, formula.ranef, sep="")), data = mdata))$coefficients)
@@ -160,13 +249,32 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
     }
     fix.all <- coxExp(uni.res2, dec = dec)
     colnames(fix.all) <- c("HR(95%CI)", "P value")
-    
+
     if (mtype == "frailty") {
       # rownames(fix.all) <- c(names(model$coefficients), "frailty")
       rownames(fix.all) <- names(model$coefficients)
     } else {
       rownames(fix.all) <- names(model$coefficients)
     }
+
+    if (use_event_filter) {
+      pattern <- paste(paste0(":", target_state_idx, "$"), collapse = "|")
+      row_keep <- grepl(pattern, rownames(fix.all))
+      kept_row_names <- rownames(fix.all)[row_keep]
+
+      if (length(kept_row_names) == 0) {
+        available_states <- paste(sprintf("%d: %s", seq_along(model_states), model_states), collapse = ", ")
+        stop(sprintf("No rows matched the supplied event_msm values. Available states: %s", available_states))
+      }
+
+      fix.all <- fix.all[kept_row_names, , drop = FALSE]
+      uni.res <- uni.res[kept_row_names, , drop = FALSE]
+      rn.uni  <- lapply(rn.uni, function(r) r[r %in% kept_row_names])
+      keep_var <- lengths(rn.uni) > 0
+      rn.uni   <- rn.uni[keep_var]
+      xf_keep  <- xf_keep[keep_var]
+      filtered_state_labels <- target_state_labels
+    }
   } else {
     countings <- length(unlist(attr(mdata[[1]], "dimnames")[2]))
     mdata2 <- cbind(matrix(sapply(mdata[, 1], `[[`, 1), ncol = countings), mdata[, -1])
@@ -198,7 +306,7 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
             out_v[i, kv2[1]] <- kv2[2]
           }
         }
-
+        
         mdata2 <- cbind(mdata2, out_v)
         
       } else {
@@ -212,11 +320,11 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
         }
       }
     }
-   
-      
     
     
-    if (!is.null(msm)) {
+    
+    
+    if (!is.null(model_states)) {
       if (is.null(data_for_univariate)) {
         baseformula <- stats::formula(paste(c(". ~ .", xf), collapse = " - "))
         unis <- lapply(xf, function(x) {
@@ -367,32 +475,103 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
       }
       
       
-      uni.res <- uni.res[rownames(uni.res) %in% rownames(mul.res), ]
-      colnames(mul.res)[ncol(mul.res)] <- "p"
+      # ==============================================================
       
-      # Determine the correct SE column from the multivariate model results
-      se_col_name <- if ("robust.se" %in% colnames(mul.res)) {
-        "robust.se"
-      } else if ("se.coef." %in% colnames(mul.res)) {
-        "se.coef."
+      if (!use_event_filter) {
+        
+        uni.res <- uni.res[rownames(uni.res) %in% rownames(mul.res), ]
+        colnames(mul.res)[ncol(mul.res)] <- "p"
+        
+        # Determine the correct SE column from the multivariate model results
+        se_col_name <- if ("robust.se" %in% colnames(mul.res)) {
+          "robust.se"
+        } else if ("se.coef." %in% colnames(mul.res)) {
+          "se.coef."
+        } else {
+          "se(coef)"
+        }
+        se_col_idx   <- which(colnames(mul.res) == se_col_name)
+        
+        # Create mul_for_exp with the correct columns for coxExp: coef, se, p
+        p_col_idx    <- which(colnames(mul.res) == "p")
+        coef_col_idx <- which(colnames(mul.res) == "coef")
+        
+        # Ensure mul_for_exp has columns in the order coxExp expects: coef, se, p
+        mul_for_exp <- mul.res[rownames(uni.res), c(coef_col_idx, se_col_idx, p_col_idx), drop = FALSE]
+        
+        fix.all <- cbind(
+          coxExp(uni.res,    dec = dec),
+          coxExp(mul_for_exp, dec = dec)
+        )
+        colnames(fix.all) <- c("crude HR(95%CI)", "crude P value", "adj. HR(95%CI)", "adj. P value")
+        rownames(fix.all) <- rownames(uni.res)
+        
       } else {
-        "se(coef)"
+        
+        pattern <- paste(paste0(":", target_state_idx, "$"), collapse = "|")
+        row_keep <- grepl(pattern, rownames(uni.res))
+        kept_row_names <- rownames(uni.res)[row_keep]
+        
+        
+        if (length(kept_row_names) == 0) {
+          available_states <- paste(sprintf("%d: %s", seq_along(model_states), model_states), collapse = ", ")
+          stop(sprintf("No rows matched the supplied event_msm values. Available states: %s", available_states))
+        }
+        
+        uni.res <- uni.res[kept_row_names, , drop = FALSE]
+        mul.res <- mul.res[kept_row_names, , drop = FALSE]
+        colnames(mul.res)[ncol(mul.res)] <- "p"
+        
+        rownames(uni.res) <- kept_row_names
+        rownames(mul.res) <- kept_row_names
+        
+        
+        se_col_name <- if ("robust.se" %in% colnames(mul.res)) {
+          "robust.se"
+        } else if ("se.coef." %in% colnames(mul.res)) {
+          "se.coef."
+        } else {
+          "se(coef)"
+        }
+        
+        mul_for_exp <- mul.res[rownames(uni.res), c("coef", se_col_name, "p"), drop = FALSE]
+        
+        fix.all <- cbind(
+          coxExp(uni.res,    dec = dec),
+          coxExp(mul_for_exp, dec = dec)
+        )
+        colnames(fix.all) <- c("crude HR(95%CI)", "crude P value", "adj. HR(95%CI)", "adj. P value")
+        rownames(fix.all) <- rownames(uni.res)
+        
+        rn.uni   <- lapply(rn.uni, function(r) r[r %in% kept_row_names])
+        keep_var <- lengths(rn.uni) > 0
+        rn.uni   <- rn.uni[keep_var]
+        xf_keep  <- xf_keep[keep_var]
+        filtered_state_labels <- target_state_labels
+        
+        
+        # rn.uni_filtered <- rn.uni
+        # rownames(fix.all) <- kept_row_names
+        if (length(target_state_idx) == 1) {
+            suffix_pattern <- paste0(":", target_state_idx, "$")
+            row_map_vals <- sub(suffix_pattern, "", kept_row_names)
+            idx_simple <- !grepl(":", row_map_vals)
+            row_map_vals[idx_simple] <- sub("_[0-9]+$", "", row_map_vals[idx_simple])
+            row_map <- setNames(row_map_vals, kept_row_names)
+
+            rownames(fix.all) <- row_map[rownames(fix.all)]
+            rn.uni <- lapply(rn.uni, function(r) {
+              mapped <- row_map[r]
+              mapped <- mapped[!is.na(mapped)]
+              mapped
+            })
+          } else {
+            rownames(fix.all) <- kept_row_names
+          }
+          rn.uni_filtered <- rn.uni
       }
-      se_col_idx <- which(colnames(mul.res) == se_col_name)
       
-      # Create mul_for_exp with the correct columns for coxExp: coef, se, p
-      p_col_idx <- which(colnames(mul.res) == "p")
-      coef_col_idx <- which(colnames(mul.res) == "coef")
-      
-      # Ensure mul_for_exp has columns in the order coxExp expects: coef, se, p
-      mul_for_exp <- mul.res[rownames(uni.res), c(coef_col_idx, se_col_idx, p_col_idx), drop = FALSE]
-      
-      fix.all <- cbind(
-        coxExp(uni.res,    dec = dec),
-        coxExp(mul_for_exp, dec = dec)
-      )      
-      colnames(fix.all) <- c("crude HR(95%CI)", "crude P value", "adj. HR(95%CI)", "adj. P value")
-      rownames(fix.all) <- rownames(uni.res)
+
     } else {
       if (is.null(data_for_univariate)) {
         basemodel <- update(model, stats::formula(paste(c(". ~ .", xf), collapse = " - ")), data = mdata2)
@@ -590,41 +769,85 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
   fix.all.list <- lapply(seq_along(xf_keep), function(x) {
     fix.all[rownames(fix.all) %in% rn.uni[[x]], ]
   })
-  varnum.mfac <- which(lapply(fix.all.list, length) > ncol(fix.all))
+  varnum.mfac_candidates <- which(lapply(fix.all.list, length) > ncol(fix.all))
+  categorical_vars_keep <- intersect(xf_keep, categorical_vars)
+  varnum.mfac <- varnum.mfac_candidates[xf_keep[varnum.mfac_candidates] %in% categorical_vars_keep]
   lapply(varnum.mfac, function(x) {
     fix.all.list[[x]] <<- rbind(rep(NA, ncol(fix.all)), fix.all.list[[x]])
   })
   fix.all.unlist <- Reduce(rbind, fix.all.list)
   
-  rn.list <- lapply(seq_along(xf_keep), function(x) {
-    rownames(fix.all)[rownames(fix.all) %in% rn.uni[[x]]]
-  })
-  varnum.2fac <- which(lapply(xf, function(x) {
-    length(sapply(mdata, levels)[[x]])
-  }) == 2)
-  lapply(varnum.2fac, function(x) {
-    rn.list[[x]] <<- paste(xf[x], ": ", levels(mdata[, xf[x]])[2], " vs ", levels(mdata[, xf[x]])[1], sep = "")
-  })
-  lapply(varnum.mfac, function(x) {
-    if (grepl(":", xf[x])) {
-      a <- unlist(strsplit(xf[x], ":"))[1]
-      b <- unlist(strsplit(xf[x], ":"))[2]
-      
-      if (a %in% xf && b %in% xf) {
-        ref <- paste0(a, levels(mdata[, a])[1], ":", b, levels(mdata[, b])[1])
-        rn.list[[x]] <<- c(paste(xf[x], ": ref.=", ref, sep = ""), gsub(xf[x], "   ", rn.list[[x]]))
+  # if (!is.null(model_states) && use_event_filter && !is.null(rn.uni_filtered)) {
+  #   rn.list <- rn.uni_filtered
+  # } else {
+  #   rn.list <- lapply(seq_along(xf_keep), function(x) {
+  #     rownames(fix.all)[rownames(fix.all) %in% rn.uni[[x]]]
+  #   })
+  #   varnum.2fac <- which(lapply(xf, function(x) {
+  #     length(sapply(mdata, levels)[[x]])
+  #   }) == 2)
+  #   lapply(varnum.2fac, function(x) {
+  #     rn.list[[x]] <<- paste(xf[x], ": ", levels(mdata[, xf[x]])[2], " vs ", levels(mdata[, xf[x]])[1], sep = "")
+  #   })
+  #   lapply(varnum.mfac, function(x) {
+  #     if (grepl(":", xf[x])) {
+  #       a <- unlist(strsplit(xf[x], ":"))[1]
+  #       b <- unlist(strsplit(xf[x], ":"))[2]
+  # 
+  #       if (a %in% xf && b %in% xf) {
+  #         ref <- paste0(a, levels(mdata[, a])[1], ":", b, levels(mdata[, b])[1])
+  #         rn.list[[x]] <<- c(paste(xf[x], ": ref.=", ref, sep = ""), gsub(xf[x], "   ", rn.list[[x]]))
+  #       } else {
+  #         rn.list[[x]] <<- c(paste(xf[x], ": ref.=NA", model$xlevels[[xf[x]]][1], sep = ""), gsub(xf[x], "   ", rn.list[[x]]))
+  #       }
+  #     } else {
+  #       rn.list[[x]] <<- c(paste(xf[x], ": ref.=", levels(mdata[, xf[x]])[1], sep = ""), gsub(xf[x], "   ", rn.list[[x]]))
+  #     }
+  #   })
+  # }
+  
+    rn.list <- lapply(seq_along(xf_keep), function(x) {
+      rownames(fix.all)[rownames(fix.all) %in% rn.uni[[x]]]
+    })
+    varnum.2fac <- which(lapply(xf, function(x) {
+      length(sapply(mdata, levels)[[x]])
+    }) == 2)
+    lapply(varnum.2fac, function(x) {
+      rn.list[[x]] <<- paste(xf[x], ": ", levels(mdata[, xf[x]])[2], " vs ", levels(mdata[, xf[x]])[1], sep = "")
+    })
+    lapply(varnum.mfac, function(x) {
+      if (grepl(":", xf[x])) {
+        a <- unlist(strsplit(xf[x], ":"))[1]
+        b <- unlist(strsplit(xf[x], ":"))[2]
+
+        if (a %in% xf && b %in% xf) {
+          ref <- paste0(a, levels(mdata[, a])[1], ":", b, levels(mdata[, b])[1])
+          rn.list[[x]] <<- c(paste(xf[x], ": ref.=", ref, sep = ""), rn.list[[x]])
+        } else {
+          rn.list[[x]] <<- c(paste(xf[x], ": ref.=NA", model$xlevels[[xf[x]]][1], sep = ""), rn.list[[x]])
+        }
       } else {
-        rn.list[[x]] <<- c(paste(xf[x], ": ref.=NA", model$xlevels[[xf[x]]][1], sep = ""), gsub(xf[x], "   ", rn.list[[x]]))
+        rn.list[[x]] <<- c(paste(xf[x], ": ref.=", levels(mdata[, xf[x]])[1], sep = ""), rn.list[[x]])
       }
-    } else {
-      rn.list[[x]] <<- c(paste(xf[x], ": ref.=", levels(mdata[, xf[x]])[1], sep = ""), gsub(xf[x], "   ", rn.list[[x]]))
-    }
-  })
+    })
+    
+    # if (!is.null(model_states) && use_event_filter && !is.null(rn.uni_filtered)) {
+    #   rn.list <- rn.uni_filtered
+    
+    # if(!is.null(model_states) && length(event_msm) == length(model_states)-1 ){
+    #   rownames(fix.all.unlist) <- unlist(rn.list)
+    # }
+  
+  
+  
   if (class(fix.all.unlist)[1] == "character") {
     fix.all.unlist <- t(data.frame(fix.all.unlist))
   }
-  if (is.null(msm)) {
-    rownames(fix.all.unlist) <- unlist(rn.list)
+  rn_vector <- unlist(rn.list)
+  if (length(rn_vector) == nrow(fix.all.unlist)) {
+    rownames(fix.all.unlist) <- rn_vector
+  } else if (is.null(model_states)) {
+    rownames(fix.all.unlist) <- rn_vector
   }
   
   pv.colnum <- which(colnames(fix.all.unlist) %in% c("P value", "crude P value", "adj. P value"))
@@ -681,11 +904,13 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
   } else if (mtype == "frailty") {
     intro <- paste("Frailty", intro, "- Group", cvname)
   }
-  if (!is.null(msm)) {
-    states <- paste(sapply(seq_along(model$states), function(i) paste(i, model$states[i], sep = ": ")), collapse = ", ")
+  if (!is.null(model_states)) {
+    states <- paste(sapply(seq_along(model_states), function(i) paste(i, model_states[i], sep = ": ")), collapse = ", ")
     intro[2] <- paste("states", states)
+    if (!is.null(filtered_state_labels)) {
+      intro <- c(intro, paste("filtered states:", paste(filtered_state_labels, collapse = ", ")))
+    }
   }
-  states <- model$states
   
   var.names0 <- attr(model$terms, "term.labels")
   if (length(grep("strata", var.names0)) > 0) {
@@ -697,4 +922,4 @@ cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL, pcut.univariate
   } else {
     return(list(table = fix.all.unlist, ranef = ranef.mat, metric = metric.mat, caption = intro))
   }
-}
\ No newline at end of file
+}
